require_relative 'learning_diary.rb'
require_relative 'views/collater_console_view.rb'

# collection class for Learning Diarys / logs
class Logs
  include Enumerable

  # iterates through every file in the data directory and parses with
  #  LearningDiary
  def initialize
    @logs = []
  end

  # Adds either a file to logs or a directory to logs
  # @param [String] file_or_directory A string containing the name of a
  #  file or directory
  def add(file_or_directory)
    log_directory(file_or_directory) if File.directory?(file_or_directory)
    log_file(file_or_directory) if File.file?(file_or_directory)
  end

  # Implements Enumerable by iterating through each learning diary
  def each(&block)
    @logs.each { |p| block.call(p) }
  end

  private

  # Adds a file to logs
  # @param [String] file A string containing the name of a file
  def log_file(file)
    @logs << LearningDiary.new(file)
  end

  # Adds a directory to logs
  # @param [String] dir A string containing the name of a directory
  def log_directory(dir)
    Dir[dir + '/**/*.txt'].each { |file| log_file(file) }
  end
end

# controller class for the diaries/logs collection
class Docfolio
  def initialize
    @logs = Logs.new
    @view = CollaterConsoleView.new
  end

  # Creates a portfolio
  # @param [String] portfolio_file_or_directory A portfolio file or a
  #  name of a directory containing portfolio files including files
  #  within subdirectories.
  def self.create(portfolio_file_or_directory)
    logs = Logs.new
    logs.add(portfolio_file_or_directory)
    CollaterConsoleView.new.print_logs(logs)
  end

  # print all parsed logs to the console
  def print_logs
    @view.print_logs(@logs)
  end
end
