# parent view class
class View
  protected

  def rm_colour(str)
    str.gsub(/\e.*?m/, '')
  end
end

class ConsoleView < View
  protected

  OUTPUT_WIDTH = 80

  def print_array_section(title, body, list)
    return '' if body.empty?
    if list
      to_console("#{title}") + "\n\n" +
      to_console("• #{body.join("\n\n• ")}\n\n", 2, 4)
    else
      to_console("#{title.gsub(/ $/, '')}") +
      to_console("#{body.join}\n\n").gsub(/ $/, '')
    end
  end

  def section_str(title, body = nil, list = false)
    return '' if body.nil?
    case body
    when Array
      return '' if body.empty?
      print_array_section(title, body, list)
    when String
      return '' if body.strip == ''
      "#{title}\n\n#{body}"
    end
  end

  def line(char)
    ' ' * 27 + char * 18 + "\n"
  end

  def sub_total_line
    line('─')
  end

  def total_line
    line('═')
  end

  def mins2hour(mins)
    hours = (mins.to_r / 60).to_i
    mins -= hours * 60
    hours == 1 ? h_str = 'hour' : h_str = 'hours'
    mins == 1 ? m_str = 'min' : m_str = 'mins'
    "#{hours}".rjust(30) + " #{h_str}".ljust(6) +
    "#{mins.to_i}".rjust(3) + " #{m_str}\n"
  end

  # prints the word to the console
  def word2console(word, x_pos, margin)
    str = ''
    length = 1 + rm_colour(word).length
    if x_pos + length >= OUTPUT_WIDTH
      str += "\n" + (' ' * margin)
      x_pos = margin + length
    else
      x_pos += length
    end
    str != '' ? debug = true : debug = false
    str += word
    [x_pos, str]
  end

  def to_console(str, first_line_margin = 0, margin = 0)
    return_str = ''
    str.each_line do |line|
      if line == "\n"
        return_str += "\n\n"
        next
      end
      return_str += ' ' * first_line_margin
      x_pos = first_line_margin
      words = line.split(' ')
      words.each do |word|
        x_pos, this_str = word2console(word, x_pos, margin)
        return_str += this_str + ' '
      end
    end
    return_str
  end
end
